; vga_static.asm - generate an static VGA picture
; author Neil Franklin, last modification 2008.09.19

; need fuses, fast external quarz osc, max stable startup
; FF: 7:BODLVL=1, 6:BOD=1 (disable), 54:SUT=11, 3210:CKSEL=1111 (ext resonator)
; D9: 7:OCDEN=1 (dis), 6:JTAGEN=1 (dis), 5:SPIEN=0 (en), 4:CKOPT=1
;     3:EESAVE=1 (not preserve), 21:BOOTSZ=00, 0:BOOTRST=1 

; PORTC is connected to an VGA HD15 adapter, 3*2bit DACs and 2 TTLs
;   bit0+1 blue DAC, bit2+3 green DAC, bit4+5 red DAC
;   bit6 hsync TTL, bit7 vsync TTL


  .include "m32def.inc"

  .cseg
  .org $0000

  jmp RESET
  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI

  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI

  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI

  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI

  jmp RETI


; get interrupts ready, despite not used
RETI:
  reti


; get stack ready, despite not used
RESET:
  ldi TL,low(MAXRAM)
  ldi TH,high(MAXRAM)
  out SPL,TL
  out SPH,TH
  sei

; constants for  out PORTC,colour
COLOURS:
  .def BLK = R8
  ldi TL,0x00
  mov BLK,TL
  .def BLU = R9
  ldi TL,0x03
  mov BLU,TL
  .def GRN = R10
  ldi TL,0x0C
  mov GRN,TL
  .def CYN = R11
  ldi TL,0x0F
  mov CYN,TL
  .def RED = R12
  ldi TL,0x30
  mov RED,TL
  .def MAG = R13
  ldi TL,0x33
  mov MAG,TL
  .def YEL = R14
  ldi TL,0x3C
  mov YEL,TL
  .def WHT = R15
  ldi TL,0x3F
  mov WHT,TL

; constants for  out PORTC,sync
SYNC:
  .def HOR = R5
  ldi TL,0x40
  mov HOR,TL
  .def VER = R6
  ldi TL,0x80
  mov VER,TL
  .def VHR = R7
  ldi TL,0xC0
  mov VHR,TL

; get PortC ready
DDR:
  ldi TL,0xFF
  out DDRC,TL

; no picture at the moment
INIT:
  out PORTC,BLK

; loop infinitely through drawing frames
LOOP:
    call FRAME    ;   4 + (449*600+583) clocks, 13 to spare
    nop           ;   10 clocks
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    jmp LOOP      ;   3 clocks


; draw frame, in 12+25 + (25*16=)400 + 12 = 449 lines
;   + 4+575 + 4 = 583 clocks (of 450 lines of 600 clocks)
;   start with retrace pulse, as first deviation from all zero
;   end with 1st cycle of last blank before next pulse, rest f callers control
FRAME:
FRPULSE:          ; must be 12 lines
  ldi TH,12       ; 1clock
FRPLOOP:
    call VRLINE   ;   4 + 575 = 579 clocks, 21 to spare
    ldi TL,3      ;   1 clock
    call WAIT     ;   4 + (3 * 3 + 3) = 16 clocks
    nop           ;   1 clock
    dec TH        ;   1 clock
    brne FRPLOOP  ;   2 clocks when taken, 1 when not

FRBLANK:          ; must be 25 lines
  ldi TH,25       ; 1clock
FRBLOOP:
    call BLLINE   ;   4 + 575 = 579 clocks, 21 to spare
    ldi TL,3      ;   1 clock
    call WAIT     ;   4 + (3 * 3 + 2) = 16 clocks
    nop           ;   1 clock
    dec TH        ;   1 clock
    brne FRBLOOP  ;   2 clocks when taken, 1 when not

FRDRAW:           ; must be 400 pixel lines

FRSOLID:          ; 5*16=80 solid block
  ldi TH,80       ; 1clock
FRSLOOP:
    call PXLINE   ;   4 + 575 = 579 clocks, 21 to spare
    ldi TL,3      ;   1 clock
    call WAIT     ;   4 + (3 * 3 + 3) = 16 clocks
    nop           ;   1 clock
    dec TH        ;   1 clock
    brne FRSLOOP  ;   2 clocks when taken, 1 when not

  ; remaining 400-80=320 pixel lines

FRSTRIPE:         ; 5*16=80 striped block
  ldi TH,40       ; 1clock
FRSTLOOP:
    call PXLINE   ;   4 + 575 = 579 clocks, 21 to spare
    ldi TL,4      ;   1 clock
    call WAIT     ;   4 + (4 * 3 + 3) = 19 clocks
    nop           ;   1 clock
    call BLLINE   ;   4 + 575 = 579 clocks, 21 to spare
    ldi TL,3      ;   1 clock
    call WAIT     ;   4 + (3 * 3 + 3) = 16 clocks
    nop           ;   1 clock
    dec TH        ;   1 clock
    brne FRSTLOOP ;   2 clocks when taken, 1 when not

  ; remaining 320-80=240 pixel lines

FRBLACK:          ; 5*16=80 black block, same as solid block
  ldi TH,80       ; 1clock
FRHLOOP:
    call BLLINE
    ldi TL,3
    call WAIT
    nop
    dec TH
    brne FRHLOOP

  ; remaining 240-80=160 pixel lines

FRSTRIPE2:        ; 5*16=80 striped block
  ldi TH,40
FRST2LOOP:
    call SHLINE
    ldi TL,4
    call WAIT
    nop
    call BLLINE
    ldi TL,3
    call WAIT
    nop
    dec TH
    brne FRST2LOOP

  ; remaining 160-80=80 pixel lines

FRSOLID2:         ; 5*16=80 solid block
  ldi TH,80
FRS2LOOP:
    call SHLINE
    ldi TL,3
    call WAIT
    nop
    dec TH
    brne FRS2LOOP

  ; remaining 80-80=0 pixel lines, all pixel lines used up

FREND:            ; must be 12 of 13 lines
  ldi TH,12       ; 1clock
FRELOOP:
    call BLLINE   ;   4 + 575 = 579 clocks, 21 to spare
    ldi TL,3      ;   1 clock
    call WAIT     ;   4 + (3 * 3 + 3) = 16 clocks
    nop           ;   1 clock
    dec TH        ;   1 clock
    brne FRELOOP  ;   2 clocks when taken, 1 when not

                  ; 13th line, last one, no wait, give rest of time to caller
  call BLLINE     ; 4 + 579 = 579 clocks, 21 to spare
  ret             ; 4 clocks


; draw vertical retrace line
;   start with horizontal retrace pulse, as first deviation from all zero
;   end with 1st cycle of blank before next pulse, rest for callers control
;   uses 30+60 + 480 + 1 + 4 = 575 clocks (of 600 in line)
VRLINE:
VRPULSE:          ; must be 30 clocks, V = 1, H = 1, color = BLK
  out PORTC,VHR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

VRBLANK:          ; must be 60+480 = 540 clocks, V = 1, H = 0, color = BLK
  out PORTC,VER   ; 1 clock
  ldi TL,177      ; 1 clock
  call WAIT       ; 4 + (177 * 3 + 3) = 538 clocks

VREND:
  out PORTC,VER   ; 1 clock, no signal effect, just for regular timing
                  ; no waiting, give rest of time to caller
  ret             ; 4 clocks


; draw blank line
;   same as VRLINE but without vertical retrace signal active
;   uses 30+60 + 480 + 1 + 4 = 575 clocks (of 600 in line)
BLLINE:
BLPULSE:          ; must be 30 clocks, V = 0, H = 1, color = BLK
  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

BLBLANK:          ; must be 60+480 = 540 clocks, V = 0, H = 0, color = BLK
  out PORTC,BLK   ; 1 clock
  ldi TL,177      ; 1 clock
  call WAIT       ; 4 + (177 * 3 + 3) = 538 clocks

BLEND:
  out PORTC,BLK   ; 1 clock, no signal effect, just for regular timing
                  ; no waiting, give rest of time to caller
  ret             ; 4 clocks


; draw pixel line
;   start with horizontal retrace pulse, as first deviation from all zero
;   end with 1st cycle of blank before next pulse, rest for callers control
;   uses 30+60 + 480 + 1 + 4 = 575 clocks (of 600 in line)
PXLINE:
PXPULSE:          ; must be 30 clocks, V = 0, H = 1, color = BLK
  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

PXBLANK:          ; must be 60 clocks, V = 0, H = 0, color = BLK
  out PORTC,BLK   ; 1 clock
  ldi TL,17       ; 1 clock
  call WAIT       ; 4 + (17 * 3 + 3) = 58 clocks

PXDRAW:           ; must be 480 pixel clocks, V = 0, H = 0, color = pixel

PXWB:             ; 6 widening white/black stripes
  out PORTC,WHT   ; 2*1 = 2
  out PORTC,BLK

  out PORTC,WHT   ; + 2*2 = 6
  nop
  out PORTC,BLK
  nop

  out PORTC,WHT   ; + 2*4 = 14
  nop
  nop
  nop
  out PORTC,BLK
  nop
  nop
  nop

  out PORTC,WHT   ; + 2*8 = 30
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  out PORTC,BLK
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  ; + 2*16=62

  out PORTC,WHT   ; 1 clock
  ldi TL,2        ; 1 clock
  call WAIT       ; 4 + (2 * 3 + 3) = 13 clocks
  nop             ; 1 clock
  out PORTC,BLK
  ldi TL,2
  call WAIT 
  nop

  ; + 2*32=126

  out PORTC,WHT   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks
  nop             ; 2 clocks
  nop
  out PORTC,BLK
  ldi TL,7
  call WAIT 
  nop
  nop

  ; remaining 480-126=354 pixel clocks

PWRAINBOW:        ; 4 widening white/black stripes
  out PORTC,RED   ; 7*1 = 7
  out PORTC,YEL
  out PORTC,GRN
  out PORTC,CYN
  out PORTC,BLU
  out PORTC,MAG
  out PORTC,BLK

  out PORTC,RED   ; + 7*2 = 21
  nop
  out PORTC,YEL
  nop
  out PORTC,GRN
  nop
  out PORTC,CYN
  nop
  out PORTC,BLU
  nop
  out PORTC,MAG
  nop
  out PORTC,BLK
  nop

  out PORTC,RED   ; + 7*4 = 49
  nop
  nop
  nop
  out PORTC,YEL
  nop
  nop
  nop
  out PORTC,GRN
  nop
  nop
  nop
  out PORTC,CYN
  nop
  nop
  nop
  out PORTC,BLU
  nop
  nop
  nop
  out PORTC,MAG
  nop
  nop
  nop
  out PORTC,BLK
  nop
  nop
  nop

  out PORTC,RED   ; + 7*8 = 105
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  out PORTC,YEL
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  out PORTC,GRN
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  out PORTC,CYN
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  out PORTC,BLU
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  out PORTC,MAG
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  out PORTC,BLK
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  ; remaining 354-105=249 pixel clocks

PXSEPAR:          ; white/black separator 9 clocks
  out PORTC,WHT   ; 9*1 clocks
  out PORTC,BLK
  out PORTC,WHT
  out PORTC,BLK
  out PORTC,WHT
  out PORTC,BLK
  out PORTC,WHT
  out PORTC,BLK
  out PORTC,WHT

  ; remaining 249-9=240 pixel clocks

PXRGB:            ; all 8 RGB colours, each 240/8=30 clocks
  out PORTC,BLK   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks
  out PORTC,BLU
  ldi TL,7
  call WAIT
  out PORTC,GRN
  ldi TL,7
  call WAIT
  out PORTC,CYN
  ldi TL,7
  call WAIT
  out PORTC,RED
  ldi TL,7
  call WAIT
  out PORTC,MAG
  ldi TL,7
  call WAIT
  out PORTC,YEL
  ldi TL,7
  call WAIT
  out PORTC,WHT
  ldi TL,7
  call WAIT

  ; remaining 240-240=0 pixel clocks, all pixel clocks used up

PXEND:
  out PORTC,BLK   ; 1 clock, start the blank before the next pulse
                  ; no waiting, give rest of time to caller
  ret             ; 4 clocks


; draw shades line
;   start with horizontal retrace pulse, as first deviation from all zero
;   end with 1st cycle of blank before next pulse, rest for callers control
;   uses 30+60 + 480 + 1 + 4 = 575 clocks (of 600 in line)
SHLINE:
SHPULSE:          ; must be 30 clocks, V = 0, H = 1, color = BLK
  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

SHBLANK:          ; must be 60 clocks, V = 0, H = 0, color = BLK
  out PORTC,BLK   ; 1 clock
  ldi TL,17       ; 1 clock
  call WAIT       ; 4 + (17 * 3 + 3) = 58 clocks

SHDRAW:           ; must be 480 pixel clocks, V = 0, H = 0, color = pixel

SHWB:             ; 16 3+3clock wide white/black stripes

  out PORTC,WHT   ; 1 clock
  nop             ; 1 clock
  ldi TL,15       ; 1 clock

SHWLOOP:
    out PORTC,BLK ;   1 clock
    nop           ;   1 clock
    dec TL        ;   1 clock, precomputed

    out PORTC,WHT ;   1 clock
    brne SHWLOOP  ;   2 clocks when taken, 1 when not
  nop             ; 1 clock

  out PORTC,BLK   ; 1 clock

  ldi AL,0x00     ; 1 clock before out, for RRGGBB, black to white direction
  ldi TL,0x3F     ; 1 clock before out, pixel count

  ; remaining 480-96=384 pixel clocks

SHRRGGBB:         ; all 64 RRGGBB colours, each 384/64=6 clocks
SHRLOOP:          ; colours 0x00..0x3E
    out PORTC,AL  ;   1 clock
    inc AL        ;   1 clock
    nop           ;   1 clock
    dec TL        ;   1 clock
    brne SHRLOOP  ;   2 clocks when taken, 1 when not
  nop             ; 1 clock

  out PORTC,AL    ; 1 clock, (last) colour 0x3F
  nop             ; 5 clocks
  nop
  nop
  nop
  nop

  ; remaining 384-384=0 pixel clocks, all pixel clocks used up

SHEND:
  out PORTC,BLK   ; 1 clock, start the blank before the next pulse
                  ; no waiting, give rest of time to caller
  ret             ; 4 clocks


; wait a while doing nothing, spinloop
;   uses TL*3-1+4 = TL*3+3 clocks, for TL=[1..255,0] (where 0=256)
;   min 1*3-1+4=11 clocks for TL=1, max 256*3-1+4=776 clocks for TL=0
WAIT:
WLOOP:
    dec TL        ;   1 clock
    brne WLOOP    ;   2 clocks when taken, 1 when not
  ret             ; 4 clocks
