; vga_text_static.asm - generate an static VGA text
; author Neil Franklin, last modification 2008.09.27

; need fuses, fast external quarz osc, max stable startup
; FF: 7:BODLVL=1, 6:BOD=1 (disable), 54:SUT=11, 3210:CKSEL=1111 (ext resonator)
; D9: 7:OCDEN=1 (dis), 6:JTAGEN=1 (dis), 5:SPIEN=0 (en), 4:CKOPT=1
;     3:EESAVE=1 (not preserve), 21:BOOTSZ=00, 0:BOOTRST=1 

; PORTC is connected to an VGA HD15 adapter, 3*2bit DACs and 2 TTLs
;   bit0+1 blue DAC, bit2+3 green DAC, bit4+5 red DAC
;   bit6 hsync TTL, bit7 vsync TTL


  .include "m32def.inc"

  .cseg
  .org $0000

  jmp RESET
  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI

  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI

  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI

  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI

  jmp RETI


; get interrupts ready, despite not used
RETI:
  reti


; get stack ready, and interrupts despite not used
RESET:
  ldi TL,low(MAXRAM)
  ldi TH,high(MAXRAM)
  out SPL,TL
  out SPH,TH
  sei

; constant for setting up DDR register
  .equ ALL_OUT = 0xFF

; constants for setting up sync and colour regs
  ; sync pulse patterns
  .equ NO_SYNC  = 0x00
  .equ HOR_SYNC = 0x40
  .equ VER_SYNC = 0x80
  .equ H_V_SYNC = 0xC0
  ; colours
  .equ BLACK   = 0x00
  .equ BLUE    = 0x03
  .equ GREEN   = 0x0C
  .equ CYAN    = 0x0F
  .equ RED     = 0x30
  .equ MAGENTA = 0x33
  .equ YELLOW  = 0x3C
  .equ WHITE   = 0x3F

; set up sync and colour patterns, for  out PORTC,value-reg
  .def NONE = R15
  ldi TL,NO_SYNC
  mov NONE,TL
  .def HOR = R14
  ldi TL,HOR_SYNC
  mov HOR,TL
  .def VER = R13
  ldi TL,VER_SYNC
  mov VER,TL
  .def H_V = R12
  ldi TL,H_V_SYNC
  mov H_V,TL
  .def BACK = R11
  ldi TL,BLUE
  mov BACK,TL
  .def FORE = R10
  ldi TL,YELLOW
  mov FORE,TL

; get PortC quiet and ready
  out PORTC,NONE
  ldi TL,ALL_OUT
  out DDRC,TL

; set up frame buffer
  call INBUF

; loop infinitely through drawing frames
LOOP:
    call FRAME    ;   4 + (449*600+583) clocks, 13 to spare
    nop           ;   10 clocks
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    jmp LOOP      ;   3 clocks


; define frame buffer
  .equ VIDCOLS  = 40
  .equ VIDCHARS = 41 ; colums + 1, including abort pseudo-char
  .equ VIDROWS  = 25

  ; starts from begin of SRAM, as nothing else in SRAM, exept stack at end
  ;  use VIDROWS*VIDCHARS, 25*41=1025, of 2048bytes
  .equ VIDMEM   = MINRAM

  ; character codes used
  .equ ABORT    = 0x07 ; leave 0..6=7 times 128instr/256bytes space before font
  .equ FIRST    = 0x08 ; support font with chars from 08 on
  .equ RANGE    = 0x03 ; how many characters do we have currently, 0..3

; initialise frame buffer
INBUF:
  ; get memory for video frame buffer
  ldi XL,low(VIDMEM)
  ldi XH,high(VIDMEM)

  ; for all frame rows
  ldi TH,VIDROWS
INRLOOP:

    ldi AL,VIDROWS ;   start with first char number derived line number
    sub AL,TH      ;   TH=VIDROWS..1, need 0..(VIDROWS-1)
    andi AL,RANGE  ;   cut down to range 0..3
    ori AL,FIRST   ;   add base char code

    ; for all frame columns, place repeatedly the 4 characters
    ldi TL,VIDCOLS
INCLOOP:

      st X+,AL
      inc AL
      andi AL,RANGE  ;   cut down to range 0..3
      ori AL,FIRST   ;   add base char code

      dec TL
    brne INCLOOP
    ldi AL,ABORT ; pseudo-char for abort
    st X+,AL

    dec TH
  brne INRLOOP
  ret


; draw frame
;   start with ver (+ hor) retrace pulse, as first/only deviation from all zero
;   end with 1st cycle of last blank before next pulse, rest f callers control
;   uses 12+25 + (25*16=)400 + 12(of13) = 449(of450) lines, each 600 clocks
;     + 4+575 + 4 = 583(of600) remaining clocks
FRAME:
  .def FRLINE = R16
  ; ver retrace pulse must be 12 lines
  ldi FRLINE,12   ; 1 clock
FRPLOOP:
    call VRLINE   ;   4 + 574 = 578 clocks, 22 to spare
    ldi TL,3      ;   1 clock
    call WAIT     ;   4 + (3 * 3 + 3) = 16 clocks
    nop           ;   2 clocks
    nop
    dec FRLINE    ;   1 clock
    brne FRPLOOP  ;   2 clocks when taken, 1 when not

  ; ver retrace blank, must be 25 lines
  ldi FRLINE,25   ; 1 clock
FRBLOOP:
    call BLLINE   ;   4 + 574 = 578 clocks, 22 to spare
    ldi TL,3      ;   1 clock
    call WAIT     ;   4 + (3 * 3 + 3) = 16 clocks
    nop           ;   2 clocks
    nop
    dec FRLINE    ;   1 clock
    brne FRBLOOP  ;   2 clocks when taken, 1 when not

  ; draw screen content, must be 25 rows * 16 = 400 lines
  ldi FRLINE,VIDROWS  ; 1 clock
FRDLOOP:
    call DRROW    ;   4 + 15*600+574 = 9572 clocks, 22 to spare
    ldi TL,3      ;   1 clock
    call WAIT     ;   4 + (3 * 3 + 3) = 16 clocks
    nop           ;   2 clocks
    nop
    dec FRLINE    ;   1 clock
    brne FRDLOOP  ;   2 clocks when taken, 1 when not

  ; vertical end blank, must be 12 (of13) lines
  ldi FRLINE,12   ; 1 clock
FRELOOP:
    call BLLINE   ;   4 + 574 = 578 clocks, 22 to spare
    ldi TL,3      ;   1 clock
    call WAIT     ;   4 + (3 * 3 + 3) = 16 clocks
    nop           ;   2 clocks
    nop
    dec FRLINE    ;   1 clock
    brne FRELOOP  ;   2 clocks when taken, 1 when not

  ; 13th line, last one, no wait, give rest of time to caller
  call BLLINE     ; 4 + 579 = 579 clocks, 21 to spare
  ret             ; 4 clocks


; draw vertical retrace pulse line
;   start with horizontal retrace pulse, as first deviation from all zero
;   end with 1st cycle of blank before next pulse, rest for callers control
;   uses 30+60 + 480 + 4 = 574 clocks (of 600 in line)
VRLINE:
  ; hor retrace pulse, must be 30 clocks, V = 1, H = 1, color = BLK
  out PORTC,H_V   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  ; hor retrace blank, must be 60+480 = 540 clocks, V = 1, H = 0, color = BLK
  out PORTC,VER   ; 1 clock
  ldi TL,177      ; 1 clock
  call WAIT       ; 4 + (177 * 3 + 3) = 538 clocks

  ; hor end blank, after last pixel, no wait, give rest of time to caller
  ret             ; 4 clocks


; draw vertical retrace blank or end line
;   same as VRLINE but without vertical retrace signal active
;   uses 30+60 + 480 + 4 = 574 clocks (of 600 in line)
BLLINE:
  ; hor retrace pulse, must be 30 clocks, V = 0, H = 1, color = BLK
  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  ; hor retrace blank, must be 60+480 = 540 clocks, V = 0, H = 0, color = BLK
  out PORTC,NONE  ; 1 clock
  ldi TL,177      ; 1 clock
  call WAIT       ; 4 + (177 * 3 + 3) = 538 clocks

  ; hor end blank, after last pixel, no wait, give rest of time to caller
  ret             ; 4 clocks


; draw screen content row, = 16 lines, of which 8 are given by font
;   start with horizontal retrace pulse, as first deviation from all zero
;   end with 1st cycle of blank before next pulse, rest for callers control
;   uses 15*600 + (30+60 + 480 + 4 = 574) = 9578 clocks (of 15*600=9000
DRROW:

; line 0 - of block of 8

  ; hor retrace pulse, must be 30 clocks, V = 0, H = 1, color = BLK
  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  ; hor retrace blank, must be 60 clocks, V = 0, H = 0, color = BLK
  out PORTC,NONE  ; 1 clock
  ldi TL,11       ; 1 clock
  call WAIT       ; 4 + (11 * 3 + 3) = 40 clocks
  nop             ; 1 clock

  ; get memory for video frame buffer, while hor retrace blank, 9 clocks
  ldi XL,low(VIDMEM)  ; 1 clock
  ldi XH,high(VIDMEM) ; 1 clock
  ; index add line offset, FRLINE=VIDROWS..1, need (0..(VIDROWS-1))*VIDCHARS
  ldi TL,VIDCHARS ; 1 clock
  ldi TH,VIDROWS  ; 1 clock
  sub TH,FRLINE   ; 1 clock
  mul TL,TH       ; 2 clocks
  add XL,ML       ; 1 clock
  adc XH,MH       ; 1 clock

  ; get first character code and unthread, while hor retrace blank, 8 clocks
  ;   uses standard font unthreading technique, see there for generic case
  ;   but compressed, without intermixed pixel OUT statements
  ld ZL,X+        ; 2 clocks, next=first character, get code
  mov ZH,ZL       ; 1 clock, first character, extract top part
  andi ZH,0x3F    ; 1 clock, for ATmega32, limit 2*64 chars of each 128instr
  andi ZL,0x80    ; 1 clock, first character, extract bottom part
  ori ZL,0x00     ; 1 clock, for in section $ccl0=$cc00, as first line
  ijmp            ; 2 clocks, first character, go and draw

  ; draw, must be 40char*4pixel*3=480 clocks, V = 0, H = 0, color = pixel
  ;   for each X+ address SRAM byte one segment, until abort charcode
  ;     so no code here, all in font, until abort handler called
  ;   abort handler does always an  out PORTC,NONE  for precise line end
  ;     then jumps to one of the following continuation points:

  ; hor end blank, after last pixel, non-last line, wait for next line
DREND0:
  ; out PORTC,NONE ; 1 clock, already done before jump
  ; jmp DREND0    ; 3 clocks, already done
  ldi TL,6        ; 1 clock
  call WAIT       ; 4 + (6 * 3 + 3) = 25 clocks

  ; 2nd hor retrace pulse, must be 30 clocks, V = 0, H = 1, color = BLK
  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  ; 2nd hor retrace blank, must be 60+480+30 = 570 clocks, V = 0, H = 0, color = BLK
  out PORTC,NONE  ; 1 clock
  ldi TL,187      ; 1 clock
  call WAIT       ; 4 + (187 * 3 + 3) = 568 clocks

; line 1 - of block of 8

  ;   repeat everything, just section offset different
  out PORTC,HOR
  ldi TL,7
  call WAIT

  out PORTC,NONE
  ldi TL,11
  call WAIT

  ldi XL,low(VIDMEM)
  ldi XH,high(VIDMEM)
  ldi TL,VIDCHARS
  ldi TH,VIDROWS
  sub TH,FRLINE
  mul TL,TH
  add XL,ML
  adc XH,MH

  ld ZL,X+
  mov ZH,ZL
  andi ZH,0x3F
  andi ZL,0x80
  ori ZL,0x10     ; for in section $ccl0=$cc10, as second line
  ijmp

DREND1:
  ldi TL,6
  call WAIT

  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  out PORTC,NONE  ; 1 clock
  ldi TL,187      ; 1 clock
  call WAIT       ; 4 + (187 * 3 + 3) = 568 clocks

; line 2 - of block of 8

  ;   repeat everything, just section offset different
  out PORTC,HOR
  ldi TL,7
  call WAIT

  out PORTC,NONE
  ldi TL,11
  call WAIT

  ldi XL,low(VIDMEM)
  ldi XH,high(VIDMEM)
  ldi TL,VIDCHARS
  ldi TH,VIDROWS
  sub TH,FRLINE
  mul TL,TH
  add XL,ML
  adc XH,MH

  ld ZL,X+
  mov ZH,ZL
  andi ZH,0x3F
  andi ZL,0x80
  ori ZL,0x20     ; for in section $ccl0=$cc20, as second line
  ijmp

DREND2:
  ldi TL,6
  call WAIT

  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  out PORTC,NONE  ; 1 clock
  ldi TL,187      ; 1 clock
  call WAIT       ; 4 + (187 * 3 + 3) = 568 clocks

; line 3 - of block of 8

  ;   repeat everything, just section offset different
  out PORTC,HOR
  ldi TL,7
  call WAIT

  out PORTC,NONE
  ldi TL,11
  call WAIT

  ldi XL,low(VIDMEM)
  ldi XH,high(VIDMEM)
  ldi TL,VIDCHARS
  ldi TH,VIDROWS
  sub TH,FRLINE
  mul TL,TH
  add XL,ML
  adc XH,MH

  ld ZL,X+
  mov ZH,ZL
  andi ZH,0x3F
  andi ZL,0x80
  ori ZL,0x30     ; for in section $ccl0=$cc30, as second line
  ijmp

DREND3:
  ldi TL,6
  call WAIT

  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  out PORTC,NONE  ; 1 clock
  ldi TL,187      ; 1 clock
  call WAIT       ; 4 + (187 * 3 + 3) = 568 clocks

; line 4 - of block of 8

  ;   repeat everything, just section offset different
  out PORTC,HOR
  ldi TL,7
  call WAIT

  out PORTC,NONE
  ldi TL,11
  call WAIT

  ldi XL,low(VIDMEM)
  ldi XH,high(VIDMEM)
  ldi TL,VIDCHARS
  ldi TH,VIDROWS
  sub TH,FRLINE
  mul TL,TH
  add XL,ML
  adc XH,MH

  ld ZL,X+
  mov ZH,ZL
  andi ZH,0x3F
  andi ZL,0x80
  ori ZL,0x40     ; for in section $ccl0=$cc40, as second line
  ijmp

DREND4:
  ldi TL,6
  call WAIT

  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  out PORTC,NONE  ; 1 clock
  ldi TL,187      ; 1 clock
  call WAIT       ; 4 + (187 * 3 + 3) = 568 clocks

; line 5 - of block of 8

  ;   repeat everything, just section offset different
  out PORTC,HOR
  ldi TL,7
  call WAIT

  out PORTC,NONE
  ldi TL,11
  call WAIT

  ldi XL,low(VIDMEM)
  ldi XH,high(VIDMEM)
  ldi TL,VIDCHARS
  ldi TH,VIDROWS
  sub TH,FRLINE
  mul TL,TH
  add XL,ML
  adc XH,MH

  ld ZL,X+
  mov ZH,ZL
  andi ZH,0x3F
  andi ZL,0x80
  ori ZL,0x50     ; for in section $ccl0=$cc50, as second line
  ijmp

DREND5:
  ldi TL,6
  call WAIT

  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  out PORTC,NONE  ; 1 clock
  ldi TL,187      ; 1 clock
  call WAIT       ; 4 + (187 * 3 + 3) = 568 clocks

; line 6 - of block of 8

  ;   repeat everything, just section offset different
  out PORTC,HOR
  ldi TL,7
  call WAIT

  out PORTC,NONE
  ldi TL,11
  call WAIT

  ldi XL,low(VIDMEM)
  ldi XH,high(VIDMEM)
  ldi TL,VIDCHARS
  ldi TH,VIDROWS
  sub TH,FRLINE
  mul TL,TH
  add XL,ML
  adc XH,MH

  ld ZL,X+
  mov ZH,ZL
  andi ZH,0x3F
  andi ZL,0x80
  ori ZL,0x60     ; for in section $ccl0=$cc60, as second line
  ijmp

DREND6:
  ldi TL,6
  call WAIT

  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  out PORTC,NONE  ; 1 clock
  ldi TL,187      ; 1 clock
  call WAIT       ; 4 + (187 * 3 + 3) = 568 clocks

; line 7 - of block of 8

  ;   repeat everything, just section offset different
  out PORTC,HOR
  ldi TL,7
  call WAIT

  out PORTC,NONE
  ldi TL,11
  call WAIT

  ldi XL,low(VIDMEM)
  ldi XH,high(VIDMEM)
  ldi TL,VIDCHARS
  ldi TH,VIDROWS
  sub TH,FRLINE
  mul TL,TH
  add XL,ML
  adc XH,MH

  ld ZL,X+
  mov ZH,ZL
  andi ZH,0x3F
  andi ZL,0x80
  ori ZL,0x70     ; for in section $ccl0=$cc70, as second line
  ijmp

  ; hor end blank, after last pixel, last line, give rest of time to caller
DREND7:
  ; out PORTC,NONE ; 1 clock, already done before jump
  ; jmp DREND7    ; 3 clocks, already done
  ldi TL,6        ; 1 clock
  call WAIT       ; 4 + (6 * 3 + 3) = 25 clocks

  ; 2nd hor retrace pulse, must be 30 clocks, V = 0, H = 1, color = BLK
  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  ; 2nd hor retrace blank, must be 60+480 = 540 clocks, V = 0, H = 0, color = BLK
  out PORTC,NONE  ; 1 clock
  ldi TL,177      ; 1 clock
  call WAIT       ; 4 + (177 * 3 + 3) = 538 clocks

  ; hor end blank, after last pixel, no wait, give rest of time to caller
  ret             ; 4 clocks


; wait a while doing nothing, spinloop
;   uses TL*3-1+4 = TL*3+3 clocks, for TL=[1..255,0] (where 0=256)
;   min 1*3-1+4=11 clocks for TL=1, max 256*3-1+4=776 clocks for TL=0
WAIT:
  ; TL has already been set up
WAILOOP:
    dec TL        ;   1 clock
    brne WAILOOP  ;   2 clocks when taken, 1 when not
  ret             ; 4 clocks


; character processing/drawing routines
;   per character 8 line segments
;   per segment size 16 instructions a 2 bytes, = 256bytes/char
;   per segment time 4 pixels a 3 clocks = 12 clocks/line

; overjump program code space, results in unusable character codes
;   we currently use character 07 as first one, the abort character
;     then use  08..7F as drawing characters
;   this allows 0..6=7 * 128instr/256bytes of code

; no labels as this code is only ever reached by computed ijmp
;   same as the entire character drawing code in the font file


; pseudo-char 07, abort line drawing
  .org $0700

  out PORTC,NONE  ; 1 clock, must be here for precise end timing
  jmp DREND0      ; 3 clocks, rest in code above
  nop             ; overjump rest space of unused 16-3=13 instructions
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  jmp DREND1
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  jmp DREND2
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  jmp DREND3
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  jmp DREND4
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  jmp DREND5
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  jmp DREND6
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  jmp DREND7
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  ; space 0780 to 07FF not used presently, would be char code 87


; here then include font
;   per character 8 line segments
;   per segment size 16 instructions, a 2 bytes, = 8*16*2=256bytes/char
;     on ATmega32 with 32k this allows max 2*64 chars, minus used code space
;     so can contain 2*(64-8) characters, 08 to 3F and 88 to BF
;   per segment time 4 pixels a 3 clocks = 12 clocks/line

; characters are interleaved as:  08 88 09 89 0A 8A ... 3F BF
;   0x..3x have  .org cc00  , 8x..Bx have  .org xx80
;   because .org and ijmp use word addressing, gives 8*16instr, not 8*32byte

; for  cc = char code  and  l = 0 1 ..7 (or l = 8 9 .. F)

; .org $cc00      ; start character cc at $cc00, cc as above

; then for each line section: run 12 clocks, 16 instructions
;   4clocks write pixels with  out PORTC,xxxx (xxxx = FORE or BACK)
;   between these 4*2clocks for unthreading next character jump
;   after all fill up with nop-s

; out PORTC,xxxx  ; 1 clock, current character, pixel 1
; ld ZL,X+        ; 2 clocks, next character, get code

; out PORTC,xxxx  ; 1 clock, current character, pixel 2
; mov ZH,ZL       ; 1 clock, next character, extract top part
; andi ZH,0x3F    ; 1 clock, for ATmega32, limit 2*64 chars of each 128instr

; out PORTC,xxxx  ; 1 clock, current character, pixel 3
; andi ZL,0x80    ; 1 clock, next character, extract bottom part
; ori ZL,0xl0     ; 1 clock, for in sections $ccl0, l as above

; out PORTC,xxxx  ; 1 clock, current character, pixel 4, = between characters
; ijmp            ; 2 clocks, next character, go and draw
                  ;   continuation at that chars pixel 1, as above

; nop             ; overjump rest space of 16-10=6 instructions
; nop
; nop
; nop
; nop
; nop

  .include "vga_text_font.inc"
