; vga_text_static.asm - generate an static VGA text
; author Neil Franklin, last modification 2008.09.27

; need fuses, fast external quarz osc, max stable startup
; FF: 7:BODLVL=1, 6:BOD=1 (disable), 54:SUT=11, 3210:CKSEL=1111 (ext resonator)
; D9: 7:OCDEN=1 (dis), 6:JTAGEN=1 (dis), 5:SPIEN=0 (en), 4:CKOPT=1
;     3:EESAVE=1 (not preserve), 21:BOOTSZ=00, 0:BOOTRST=1 

; PORTC is connected to an VGA HD15 adapter, 3*2bit DACs and 2 TTLs
;   bit0+1 blue DAC, bit2+3 green DAC, bit4+5 red DAC
;   bit6 hsync TTL, bit7 vsync TTL


  .include "m32def.inc"

  .cseg
  .org $0000

  jmp RESET
  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI

  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI

  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI

  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI
  jmp RETI

  jmp RETI


; get interrupts ready, despite not used
RETI:
  reti


; get stack ready, and interrupts despite not used
RESET:
  ldi TL,low(MAXRAM)
  ldi TH,high(MAXRAM)
  out SPL,TL
  out SPH,TH
  sei

; constant for setting up DDR register
  .equ ALL_OUT = 0xFF

; constants for setting up sync and colour regs
  ; sync pulse patterns
  .equ NO_SYNC  = 0x00
  .equ HOR_SYNC = 0x40
  .equ VER_SYNC = 0x80
  .equ H_V_SYNC = 0xC0
  ; colours
  .equ BLACK   = 0x00
  .equ BLUE    = 0x03
  .equ GREEN   = 0x0C
  .equ CYAN    = 0x0F
  .equ RED     = 0x30
  .equ MAGENTA = 0x33
  .equ YELLOW  = 0x3C
  .equ WHITE   = 0x3F

; set up sync and colour patterns, for  out PORTC,value-reg
  .def NONE = R15
  ldi TL,NO_SYNC
  mov NONE,TL
  .def HOR = R14
  ldi TL,HOR_SYNC
  mov HOR,TL
  .def VER = R13
  ldi TL,VER_SYNC
  mov VER,TL
  .def H_V = R12
  ldi TL,H_V_SYNC
  mov H_V,TL
  .def BACK = R11
  ldi TL,BLUE
  mov BACK,TL
  .def FORE = R10
  ldi TL,YELLOW
  mov FORE,TL

; get PortC quiet and ready
  out PORTC,NONE
  ldi TL,ALL_OUT
  out DDRC,TL

; set up frame buffer
  call INBUF

; loop infinitely through drawing frames
LOOP:
    call FRAME    ;   4 + (449*600+583) clocks, 13 to spare
    nop           ;   10 clocks, ldi TL,1 + call WAIT only from 11 on upwards
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    jmp LOOP      ;   3 clocks


; define frame buffer
  .equ VIDCOLS  = 40 ; columns of chars in one row
  .equ VIDCHARS = 41 ; colums + 1, including abort pseudo-char
  .equ VIDROWS  = 25 ; rows of characters

  ; starts from begin of SRAM, as nothing else in SRAM, exept stack at end
  ;   uses VIDROWS*VIDCHARS, 25*41=1025, of 2048bytes
  .equ VIDMEM   = MINRAM

  ; character codes used
  .equ ABORT    = 0x07 ; leave 0..6=7 times 128instr/256bytes space before font
  .equ FIRST    = 0x08 ; support font with chars from 08 on
  .equ RANGE    = 0x03 ; how many characters do we have currently, 0..3

; initialise frame buffer
INBUF:
  ; get memory for video frame buffer
  ldi XL,low(VIDMEM)
  ldi XH,high(VIDMEM)

  ; for all frame rows
  ldi TH,VIDROWS
INRLOOP:

    ldi AL,VIDROWS ;   start with first char number derived line number
    sub AL,TH      ;   TH=VIDROWS..1, need 0..(VIDROWS-1)
    andi AL,RANGE  ;   cut down to range 0..3
    ori AL,FIRST   ;   add base char code

    ; for all frame columns, place repeatedly the 4 characters
    ldi TL,VIDCOLS
INCLOOP:

      st X+,AL
      inc AL
      andi AL,RANGE  ;   cut down to range 0..3
      ori AL,FIRST   ;   add base char code

      dec TL
      brne INCLOOP

    ldi AL,ABORT   ; pseudo-char for abort
    st X+,AL

    dec TH
    brne INRLOOP

  ret


; draw frame
;   start with ver (+ hor) retrace pulse, as first/only deviation from all zero
;   end with 1st cycle of last blank before next pulse, rest f callers control
;   uses 12+25 + (25*16=)400 + 12(of13) = 449(of450) lines, each 600 clocks
;     + 4+575 + 4 = 583(of600) remaining clocks
FRAME:
  .def FRLINE = R16
  .def FRROW  = R17

  ; vertical retrace pulse, must be 12 lines
  ldi FRLINE,12   ; 1 clock, delays everything by 1 clock, no problem
FRPLOOP:

    call VPLINE   ;   4 + 575 = 579 clocks, 21 to spare
    ldi TL,3      ;   1 clock
    call WAIT     ;   4 + (3 * 3 + 3) = 16 clocks
    nop           ;   1 clock

    dec FRLINE    ;   1 clock
    brne FRPLOOP  ;   2 clocks when taken, 1 when not

  ; vertical retrace blank, must be 24 (or 25) lines
  ldi FRLINE,25   ; 1 clock, also compensates for last brne only 1 clock
FRBLOOP:

    call VBLINE   ;   4 + 575 = 579 clocks, 21 to spare
    ldi TL,3      ;   1 clock
    call WAIT     ;   4 + (3 * 3 + 3) = 16 clocks
    nop           ;   1 clock

    dec FRLINE    ;   1 clock
    brne FRBLOOP  ;   2 clocks when taken, 1 when not

  nop             ; 1 clock, compensate for last brne only 1 clock

  ; 25th line, last one in blank, no wait, give rest of time to character setup
  ;   subtract later in draw setup, 7 clocks
  call VBLINE     ; 4 + 579 = 579 clocks, 21-7=14 to spare
  ldi TL,2        ; 1 clock
  call WAIT       ; 4 + (2 * 3 + 3) = 13 clocks
  nop             ; 1 clock

  ; characters, must be 25 rows * 16 = 384 (or 400) lines
  ldi YL,low(VIDMEM)  ; 1 clock, get frame buffer first row begin
  ldi YH,high(VIDMEM) ; 1 clock

  ldi FRROW,VIDROWS   ; 1 clock
FRRLOOP:

    ldi AL,0x00   ;   1 clock, start with first line segment offset

    ldi FRLINE,7  ;   1 clock, 7 (of 8) lines pairs per row
FRLLOOP:

      movw XH:XL,YH:YL ; 1 clock

      call CHLINE   ;   4 + 575 = 579 clocks, 21 to spare
      ldi TL,4      ;   1 clock
      call WAIT     ;   4 + (4 * 3 + 3) = 19 clocks

      movw XH:XL,YH:YL ; 1 clock

      call CHLINE   ;   4 + 575 = 579 clocks, 21 to spare
      ldi TL,2      ;   1 clock
      call WAIT     ;   4 + (2 * 3 + 3) = 13 clocks
      nop           ;   1 clock

      subi AL,-0x10 ;   1 clock, move line segment offset by 16instr/32bytes
      andi AL, 0x70 ;   1 clock, keep it in range, also after exiting loop

      dec FRLINE    ;   1 clock
      brne FRLLOOP  ;   2 clocks when taken, 1 when not
                    ;   + 1 clock movw after FRLLOOP

    ; 8th line, last one in row, no wait, give rest of time to row stepping
    nop           ; 1 clock, compensate for last brne only 1 clock
    movw XH:XL,YH:YL ; 1 clock, this is already precompensated for

    call CHLINE   ;   4 + 575 = 579 clocks, 21 to spare
    ldi TL,4      ;   1 clock
    call WAIT     ;   4 + (4 * 3 + 3) = 19 clocks

    movw XH:XL,YH:YL ; 1 clock

    call CHLINE   ;   4 + 575 = 579 clocks, 21 to spare
    ldi TL,1      ;   1 clock
    call WAIT     ;   4 + (1 * 3 + 3) = 10 clocks

    subi AL,-0x10 ;   1 clock, move line segment offset by 16instr/32bytes
    andi AL, 0x70 ;   1 clock, keep it in range, also after exiting loop

    ; next row stuff, just update frame buffer row begin
    adiw YH:YL,VIDCHARS ; 2 clocks, jump video frame buffer to next row

    dec FRROW     ;   1 clock
    brne FRRLOOP  ;   2 clocks when taken, 1 when not
                  ;   + 3 clocks ldi/ldi/movw after FRRLOOP

  ; vertical retrace end, must be 12 (of 13) lines
  ldi FRLINE,12   ; 1 clock, also compensates for last brne only 1 clock
FRELOOP:

    call VBLINE   ;   4 + 575 = 579 clocks, 21 to spare
    ldi TL,3      ;   1 clock
    call WAIT     ;   4 + (3 * 3 + 3) = 16 clocks
    nop           ;   1 clock

    dec FRLINE    ;   1 clock
    brne FRELOOP  ;   2 clocks when taken, 1 when not

  nop             ; 1 clock, compensate for last brne only 1 clock

  ; 13th line, last one in frame, no wait, give rest of time to caller
  call VBLINE     ; 4 + 579 = 579 clocks, 21 to spare
  ret             ; 4 clocks


; draw vertical retrace pulse line
;   start with horizontal retrace pulse, as first deviation from all zero
;   end with 1st cycle of blank before next pulse, rest for callers control
;   uses 30+60 + 480 + 1+4 = 575 clocks (of 600 in line)
VPLINE:

  ; hor retrace pulse, must be 30 clocks, V = 1, H = 1, color = BLK
  out PORTC,H_V   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  ; hor retrace blank, must be 60+480 = 540 clocks, V = 1, H = 0, color = BLK
  out PORTC,VER   ; 1 clock
  ldi TL,177      ; 1 clock
  call WAIT       ; 4 + (177 * 3 + 3) = 538 clocks

  ; hor end blank, after last pixel, no wait, give rest of time to caller
  out PORTC,VER   ; 1 clock, no signal effect, just for regular timing
  ret             ; 4 clocks


; draw vertical retrace blank or end line
;   parameters: TL loop count, 1..255 or 0(=256)
;   same as VPLINE but without vertical retrace signal active
;   uses 30+60 + 480 + 1+4 = 575 clocks (of 600 in line)
VBLINE:

  ; hor retrace pulse, must be 30 clocks, V = 0, H = 1, color = BLK
  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  ; hor retrace blank, must be 60+480 = 540 clocks, V = 0, H = 0, color = BLK
  out PORTC,NONE  ; 1 clock
  ldi TL,177      ; 1 clock
  call WAIT       ; 4 + (177 * 3 + 3) = 538 clocks

  ; hor end blank, after last pixel, no wait, give rest of time to caller
  out PORTC,NONE  ; 1 clock, no signal effect, just for regular timing
  ret             ; 4 clocks


; draw characters line
;   parameters: XH:XL first char SRAM address
;               AL font line section offset, 0x00, 0x10, .. 0x70
;   start with horizontal retrace pulse, as first deviation from all zero
;   end with 1st cycle of blank before next pulse, rest for callers control
;   uses 30+60 + 480 + 1+4 = 575 clocks (of 600 in line)
CHLINE:

  ; hor retrace pulse, must be 30 clocks, V = 0, H = 1, color = BLK
  out PORTC,HOR   ; 1 clock
  ldi TL,7        ; 1 clock
  call WAIT       ; 4 + (7 * 3 + 3) = 28 clocks

  ; hor retrace blank, must be 60 clocks, V = 0, H = 0, color = BLK
  ;   subtract later in hor retrace code, 8 clocks, needs still 52
  out PORTC,NONE  ; 1 clock
  ldi TL,14       ; 1 clock
  call WAIT       ; 4 + (14 * 3 + 3) = 49 clocks
  nop             ; 1 clock

  ; get first character code and unthread, in hor retrace blank, 8 clocks
  ;   uses standard font unthreading technique, see there for generic case
  ;   but compressed, without intermixed pixel OUT statements
  ; XH:XL address has already been set up by caller
  ld ZL,X+        ; 2 clocks, next=first character, get code
  mov ZH,ZL       ; 1 clock, first character, extract top part
  andi ZH,0x3F    ; 1 clock, for ATmega32, limit 2*64 chars of each 128instr
  andi ZL,0x80    ; 1 clock, first character, extract bottom part
  ; AL pattern has already been set up by caller
  or ZL,AL        ; 1 clock, add variable section offset for this line in row
  ijmp            ; 2 clocks, first character, go and draw

  ; draw, must be 40char*4pixel*3=480 clocks, V = 0, H = 0, color = pixel
  ;   for each X+ address SRAM byte indexes one segment, until abort charcode
  ;     so no code here, all in font, until abort handler called
  ;   abort handler does always  out PORTC,NONE  for safe line end, then  ret
  ; so there is no "end of line" stuff here, see character 07 below for that


; wait a while doing nothing, spinloop
;   parameters: TL loop count, 1..255 or 0(=256)
;   uses TL*3-1+4 = TL*3+3 clocks, for TL=[1..255,0]
;   min 1*3-1+4=11 clocks for TL=1, max 256*3-1+4=776 clocks for TL=0
WAIT:

  ; TL count has already been set up by caller
WAILOOP:

    dec TL        ;   1 clock
    brne WAILOOP  ;   2 clocks when taken, 1 when not

  ret             ; 4 clocks


; character processing/drawing routines
;   per character 8 line segments
;   per segment size 16 instructions a 2 bytes, = 256bytes/char
;   per segment time 4 pixels a 3 clocks = 12 clocks/line

; overjump program code space, results in unusable character codes
;   we currently use character 07 as first one, the abort character
;     then use  08..7F as drawing characters
;   this allows 0..6=7 * 128instr/256bytes of code

; no labels as this code is only ever reached by computed ijmp
;   same as the entire character drawing code in the font file


; pseudo-char 07, abort line drawing
  .org $0700

  ; hor end blank, after last pixel, no wait, give rest of time to caller
  out PORTC,NONE  ; 1 clock, switch off whatever was last background colour
  ret             ; 4 clocks
  nop             ; overjump rest space of unused 16-2=14 instructions
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  ret
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  ret
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  ret
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  ret
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  ret
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  ret
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  out PORTC,NONE
  ret
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop

  ; space 0780 to 07FF not used presently, would be char code 87


; here then include font
;   per character 8 line segments
;   per segment size 16 instructions, a 2 bytes, = 8*16*2=256bytes/char
;     on ATmega32 with 32k this allows max 2*64 chars, minus used code space
;     so can contain 2*(64-8) characters, 08 to 3F and 88 to BF
;   per segment time 4 pixels a 3 clocks = 12 clocks/line

; per character:

; .org $cco0      ; start character code cc first offset o at $cco0
;                       cc = 08..3x or 80..BF
;                         .org and ijmp word addressed, 8*16instr, not 8*32byte
;                         characters interleaved:  08 88 09 89 0A 8A ... 3F BF
;                       o = cc00 for 0x..3x and cc80 for 8x..Bx

; per line section:
;   run 12 clocks, 16 instructions
;     4clocks write pixels with  out PORTC,xxxx (xxxx = FORE or BACK)
;     between these 4*2clocks for unthreading next character jump

; out PORTC,xxxx  ; 1 clock, current character, pixel 1
; ld ZL,X+        ; 2 clocks, next character, get code

; out PORTC,xxxx  ; 1 clock, current character, pixel 2
; mov ZH,ZL       ; 1 clock, next character, extract top part
; andi ZH,0x3F    ; 1 clock, for ATmega32, limit 2*64 chars of each 128instr

; out PORTC,xxxx  ; 1 clock, current character, pixel 3
; andi ZL,0x80    ; 1 clock, next character, extract bottom part
; ori ZL,0xo0     ; 1 clock, add in section offset for this line in row
                  ;          o = 0x00 0x10 .. 0x70 or o = 0x80 0x90 .. 0xF0

; out PORTC,xxxx  ; 1 clock, current character, pixel 4, = between characters
; ijmp            ; 2 clocks, next character, go and draw
                  ;   continuation at that chars pixel 1, as above

; nop             ; overjump rest space of 16-10=6 instructions
; nop
; nop
; nop
; nop
; nop

  .include "vga_text_font.inc"
